# CLASSE COORDENADA
# A entidade de Hex
class Hex:
    def __init__(self, q: int, r: int, s: int):
        assert not (int(q) + int(r) + int(s) !=
                    0), "a soma de 'q', 'r' e 's' deve ser 0"
        self.q = q
        self.r = r
        self.s = s
        self.id = (q, r, s)

        # As direções dos vizinhos, iniciando de cima, horário
        self._direcoes = [
            (0, -1, 1),
            (1, -1, 0),
            (1, 0, -1),
            (0, 1, -1),
            (-1, 1, 0),
            (-1, 0, 1),
        ]

    def __str__(self):
        return f'{self.id}'

    def __repr__(self):
        return f'<Hexágono({int(self.q)}, {int(self.r)}, {int(self.s)})>'

    def __eq__(self, other):
        return (self.q, self.r, self.s) == (other.q, other.r, other.s)

    def __add__(self, other):
        return Hex(self.q + other.q, self.r + other.r, self.s + other.s)

    def __sub__(self, other):
        return Hex(self.q - other.q, self.r - other.r, self.s - other.s)

    def __mul__(self, valor):
        return Hex(self.q * valor, self.r * valor, self.s * valor)

    def __truediv__(self, valor):
        return Hex(self.q / valor, self.r / valor, self.s / valor)

    # PROPRIEDADES
    @property
    def comprimento(self):
        return int((abs(self.q) + abs(self.r) + abs(self.s))/2)

    # MÉTODOS
    def _direcao(self, direcao: int):
        assert direcao in range(6), 'a direção deve estar entre 0 a 5'
        return self._direcoes[direcao]

    def distancia(self, other):
        novo_hex = self - other
        return novo_hex.comprimento

    def vizinho(self, index: int):
        return self + Hex(*self._direcao(index))



from .hex import Hex
from .base import *

from math import sqrt, pi, cos, sin


ORIENTACAO_HEX = {
    'pontudo': Orientacao(sqrt(3), sqrt(3) / 2, 0, 3 / 2, sqrt(3) / 3, -1 / 3, 0, 2 / 3, 0.5),
    'plano': Orientacao(3 / 2, 0, sqrt(3) / 2, sqrt(3), 2 / 3, 0, -1 / 3, sqrt(3) / 3, 0)
}

# FUNÇÕES DE AUXÍLIO
def _ponto_hex_em_px(hex: Hex, tipo_layout: str = 'pontudo', tamanho_hex: int = 50, origem: tuple = Coordenada(0, 0)):
    orientacao = ORIENTACAO_HEX[tipo_layout]

    x = (orientacao.f0 * hex.q + orientacao.f1 * hex.r) * tamanho_hex
    y = (orientacao.f2 * hex.q + orientacao.f3 * hex.r) * tamanho_hex

    return Coordenada(x + origem.x, y + origem.y)


def _desvio_quina_hex(ponto: int, tipo_layout: str = 'pontudo', tamanho_hex: int = 50):
    orientacao = ORIENTACAO_HEX[tipo_layout]
    angulo = 2 * pi * (orientacao.angulo_inicial - ponto) / 6

    return Coordenada(tamanho_hex * cos(angulo), tamanho_hex * sin(angulo))


# FUNÇÃO PARA AS COORDENADAS DOS VERTÍCES
def vertices_hex(hex: Hex, geometria_hex: str = 'pontudo', tamanho_hex: int = 50, origem: tuple = Coordenada(0, 0)):
    """
    Pega as coordenadas dos seis pontos de um hexagono, em pixels
    """
    vertices = []
    centro = _ponto_hex_em_px(hex, geometria_hex, tamanho_hex, origem)

    for i in range(6):
        desvio = _desvio_quina_hex(i, geometria_hex, tamanho_hex)
        vertices.append((centro.x + desvio.x, centro.y + desvio.y))

    return vertices


# FUNÇÕES PARA DESENHAR A GRID
def grid_hexagonal(tamanho_grid: int):
    """
    Desenha uma grid de hexágonos no formato hexagonal, ao redor do hexágono do meio
    """
    dist = tamanho_grid
    mapa = []

    for q in range(-dist, dist + 1):
        r1 = max(-dist, -q - dist)
        r2 = min(dist, -q + dist)

        for r in range(r1, r2 + 1):
            mapa.append(Hex(q, r, -q-r))

    return mapa


def grid_retangular(qtd_horizontal: int, qtd_vertical: int, tipo_layout: str = 'pontudo'):
    """Desenha uma grid de hexágonos no formato retângular, tradicional"""
    topo, direita, baixo, esquerda = 0, qtd_horizontal, qtd_vertical, 0
    mapa = []

    if tipo_layout == 'pontudo':
        for r in range(topo, baixo + 1):
            r_margem = r // 2

            for q in range(esquerda - r_margem, direita - r_margem + 1):
                mapa.append(Hex(q, r, -q-r))

    if tipo_layout == 'plano':
        for q in range(esquerda, direita + 1):
            q_margem = q // 2

            for r in range(topo - q_margem, baixo - q_margem + 1):
                mapa.append(Hex(q, r, -q-r))

    return mapa


# FUNÇÃO PARA A ENCONTRAR UM HEXÁGONO NA GRID
def filtra_hex(grid: list, id: tuple):
    """Encontra um hexágono dentro da lista, com um valor específico, se houver"""
    lista = list(filter(lambda hex: hex.id == id, grid))

    return lista[0] if len(lista) > 0 else None




from setuptools import setup, find_packages


def le_arquivo(nome):
    with open(nome, 'r', encoding='utf-8') as file:
        return file.read()


setup(
    name="HEXaag",
    version='1.0.0',
    packages=find_packages(),
    author='Alessandro Guarita',
    description='Ferramenta para gerar grid de hexágonos, para a criação de mapas de RPG.',
    long_description=le_arquivo('README.md'),
    long_description_content_type='text/markdown',
    LICENCE='MIT',
    python_requires='>=3.11',
)



from collections import namedtuple

# TUPLAS COM NOME
Orientacao = namedtuple("Orientação", ['f0', 'f1', 'f2', 'f3', 'b0', 'b1', 'b2', 'b3', 'angulo_inicial'])
Coordenada = namedtuple("Coordenada", ['x', 'y'])
Medida = namedtuple("Medida", ['largura', 'altura'])
